import { readFile } from "fs/promises";
import { resolve as resolvePath } from "path";

import Shopify from "@shopify/shopify-api";
import { Context } from "koa";

export const TOP_LEVEL_OAUTH_COOKIE_NAME = "shopifyTopLevelOAuth"; // If this is set, then it knows to perform inline oauth
const RELATIVE_APP_BRIDGE_PATH = "../app-bridge/app-bridge@3.2.6.js";
const APP_BRIDGE_FILE_PATH = resolvePath(__dirname, RELATIVE_APP_BRIDGE_PATH); // Get global path from relative path to this module

export function shouldPerformTopLevelOAuth({ cookies }: Context) {
  return Boolean(cookies.get(TOP_LEVEL_OAUTH_COOKIE_NAME));
}

export function setTopLevelOAuthCookieValue(ctx: Context, value: string) {
  ctx.cookies.set(
    TOP_LEVEL_OAUTH_COOKIE_NAME,
    value,
    value != null ? getCookieOptions(ctx) : undefined
  );
}

function getCookieOptions(ctx: Context) {
  const { header } = ctx;
  const userAgent = header["user-agent"];
  const isChrome = userAgent && userAgent.match(/chrome|crios/i);
  let cookieOptions = {};
  if (isChrome) {
    cookieOptions = { secure: true };
  }
  return cookieOptions;
}

export async function startTopLevelOauthRedirect(ctx: Context, apiKey: string, path: string) {
  setTopLevelOAuthCookieValue(ctx, "1");
  let { query } = ctx;
  const hostName = Shopify.Context.HOST_NAME; // Use this instead of ctx.host to prevent issues when behind a proxy
  const shop = query.shop ? query.shop.toString() : "";
  const host = query.host ? query.host.toString() : "";
  const params = { shop, host };
  const queryString = new URLSearchParams(params).toString(); // Use this instead of ctx.querystring, because it sanitizes the query parameters we are using
  ctx.body = await getTopLevelRedirectScript(
    host,
    `https://${hostName}${path}?${queryString}`,
    apiKey
  );
}

async function getTopLevelRedirectScript(host: string, redirectTo: string, apiKey: string) {
  // We used to load the script from unpkg.com, but that sometimes was too slow, so we are now loading the script file directly and injecting the code.
  const appBridgeScript = await readFile(APP_BRIDGE_FILE_PATH);
  return `
    <!-- Shopify App Bridge -->
    <script type="text/javascript">${appBridgeScript}</script>
    <script type="text/javascript">
      document.addEventListener('DOMContentLoaded', function() {
        if (window.top === window.self) {
          // If the current window is the 'parent', change the URL by setting location.href
          window.location.href = "${redirectTo}";
        } else {
          // If the current window is the 'child', change the parent's URL with postMessage
          var AppBridge = window['app-bridge'];
          var createApp = AppBridge.default;
          var Redirect = AppBridge.actions.Redirect;
          var app = createApp({
            apiKey: "${apiKey}",
            host: "${encodeURI(host)}",
          });
          var redirect = Redirect.create(app);
          redirect.dispatch(Redirect.Action.REMOTE, "${redirectTo}");
        }
      });
    </script>
  `;
}
